<?php namespace Myriad\Squire;

use GuzzleHttp\Client as Guzzle;
use Myriad\Squire\Contracts\Squire;
use Ramsey\Uuid\Uuid;

class SquireClient implements Squire {

	/**
	 * @var \Myriad\Squire\SquireConfiguration
	 */
	private $config;

	/**
	 * @var \Myriad\Squire\SquireHelpers
	 */
	private $helpers;

	/**
	 * @var \GuzzleHttp\Client
	 */
	private $guzzle;

	/**
	 * @inheritdoc
	 */
	public function __construct(SquireConfiguration $config, SquireHelpers $helpers, Guzzle $guzzle) {
		$this->config = $config;
		$this->helpers = $helpers;
		$this->guzzle = $guzzle;
	}

	/**
	 * @inheritdoc
	 */
	public function get($key) {
		return $this->doRequest('GET', $key)->getBody()->detach();
	}

	/**
	 * @inheritdoc
	 */
	public function put($resource, $key) {
		$body = $this->helpers->openResource($resource);
		$this->doRequest('PUT', $key, [
			'body' => $body,
		]);
		if (is_resource($body)) {
			@fclose($body);
		}
	}

	/**
	 * @inheritdoc
	 */
	public function delete($key) {
		$this->doRequest('DELETE', $key);
	}

	/**
	 * Creates and sends an authorized request
	 *
	 * @param string $method the http method
	 * @param string $key the file key
	 * @param array $options additional guzzle options
	 *
	 * @return \GuzzleHttp\Message\FutureResponse|\GuzzleHttp\Message\ResponseInterface|\GuzzleHttp\Ring\Future\FutureInterface|null
	 */
	private function doRequest($method, $key, $options = []) {
		$defaultOptions = [
			'headers' => [
				'Authorization' => $this->getConfiguration()->getServerToken(),
			],
		];
		return $this->guzzle->request($method, $this->config->getServerUrl() . '/api/v1/files/'
			. $key, array_merge($defaultOptions, $options));
	}

	/**
	 * @inheritdoc
	 */
	public function image($key) {
		return new SquireImage($key, $this->getConfiguration());
	}

	/**
	 * @inheritdoc
	 */
	public function file($key) {
		return new SquireFile($key, $this->getConfiguration());
	}

	/**
	 * @inheritdoc
	 */
	public function getConfiguration() {
		return $this->config;
	}

	/**
	 * Generate a unique key with a given extension
	 *
	 * @param string $extension The file extension
	 * @param string $prefix A string to prefix the key with
	 *
	 * @return string
	 */
	public function generateKey($extension, $prefix = '') {
		return $prefix . Uuid::uuid4() . '.' . ltrim($extension, '.');
	}

	/**
	 * Generate a unique key from a file object
	 *
	 * @param \SplFileInfo $file The file object
	 * @param string $prefix A string to prefix the key with
	 *
	 * @return string
	 */
	public function generateKeyFromFile($file, $prefix = '') {
		return rtrim($this->generateKey($this->getExtensionFromFile($file), $prefix), '.');
	}

	/**
	 * Gets the file extension of a file
	 *
	 * @param \SplFileInfo $file the file object
	 *
	 * @return string the file extension
	 */
	private function getExtensionFromFile($file) {
		if (is_a($file, 'Symfony\Component\HttpFoundation\File\UploadedFile')) {
			if (is_callable([$file, 'getClientOriginalExtension']) && !empty($file->getClientOriginalExtension())) {
				return $file->getClientOriginalExtension();
			}
			return pathinfo($file->getClientOriginalName(), PATHINFO_EXTENSION);
		}

		return $file->getExtension();
	}

}